import React, { useState, useEffect, useCallback, useRef } from 'react';
import ReactDOM from 'react-dom/client';
import { Auth0Provider, useAuth0, withAuthenticationRequired } from '@auth0/auth0-react';

//======================================================================
// DE-STRUCTURED APPLICATION CODE
// All components, hooks, types, and services are in this single file
// to avoid browser module resolution issues in a no-build setup.
//======================================================================

// === From: types.ts ===

interface Source {
  uri: string;
  title?: string;
}

interface HeadlineDetail {
  element: string;
  text: string;
}

interface Headlines {
  title: string;
  details: HeadlineDetail[];
}

interface SearchResultItem {
  id: string;
  title: string;
  url: string;
  provider: string;
  providerId: string;
  updatedAt: string;
  headlines: Headlines;
  collection: string;
}

interface ResultsConfigs {
    headingsPerTitle: number;
    headingsPerTitleMore: number;
    headingsCurrentLimit: number;
    headingsLastLimit: number;
    titledOnlineIDDoNotShow: string[];
    headlinesShow: string[];
    headlinesFilterForScreen: string[];
    updatesShow: string[];
    headlinesPerPerHashTagMin: number;
}

interface ApiConfig {
    baseUrl: string;
    defaultXMediaWriteLocation: string;
    size: number;
    resultsConfigs: ResultsConfigs;
}

interface SearchConfig {
  appTitle: string;
  appSubtitle: string;
  searchDebounceMilliseconds: number;
  searchMode: 'mock' | 'api';
  api: ApiConfig;
  searchInput: {
    placeholder: string;
  };
  searchContext: {
    queryLabel: string;
    collectionLabel: string;
    resultsLabel: string;
  };
  results: {
    noResultsText: string;
  };
  headlines: {
    title: string;
    expandText: string;
  };
}

interface AuthConfig {
  domain: string;
  clientId: string;
}


// === From: components/icons.tsx ===

const SearchIcon: React.FC = () => (
  <svg
    xmlns="http://www.w3.org/2000/svg"
    className="h-6 w-6 text-slate-400"
    fill="none"
    viewBox="0 0 24 24"
    stroke="currentColor"
    strokeWidth={2}
  >
    <path
      strokeLinecap="round"
      strokeLinejoin="round"
      d="M21 21l-6-6m2-5a7 7 0 11-14 0 7 7 0 0114 0z"
    />
  </svg>
);

const LinkIcon: React.FC = () => (
    <svg 
        xmlns="http://www.w3.org/2000/svg" 
        className="h-5 w-5" 
        fill="none" 
        viewBox="0 0 24 24" 
        stroke="currentColor"
        strokeWidth={2}>
        <path strokeLinecap="round" strokeLinejoin="round" d="M13.828 10.172a4 4 0 00-5.656 0l-4 4a4 4 0 105.656 5.656l1.102-1.101m-.758-4.899a4 4 0 005.656 0l4-4a4 4 0 00-5.656-5.656l-1.1 1.1" />
    </svg>
);

const HeartIcon: React.FC = () => (
  <svg
    xmlns="http://www.w3.org/2000/svg"
    className="h-8 w-8 text-black"
    viewBox="0 0 20 20"
    fill="currentColor"
  >
    <path
      fillRule="evenodd"
      d="M3.172 5.172a4 4 0 015.656 0L10 6.343l1.172-1.171a4 4 0 115.656 5.656L10 17.657l-6.828-6.829a4 4 0 010-5.656z"
      clipRule="evenodd"
    />
  </svg>
);

const UserIcon: React.FC = () => (
    <svg 
      xmlns="http://www.w3.org/2000/svg" 
      className="h-5 w-5" 
      viewBox="0 0 20 20" 
      fill="currentColor">
      <path fillRule="evenodd" d="M10 9a3 3 0 100-6 3 3 0 000 6zm-7 9a7 7 0 1114 0H3z" clipRule="evenodd" />
    </svg>
);

// === From: services/mockingResults.ts ===
const MOCK_DATA: any[] = [
  {
    "titledOnlineID": "perfectjourney_perfect_reservation_com",
    "HCA": "unknown",
    "headlines": {
        "0": { "textContent": "Sari la conținut", "className": "skip-link screen-reader-text", "tagName": "A" },
        "1": { "textContent": "Perfect Reservation Logo", "className": "custom-logo-link", "tagName": "A", "href": "https://perfect-reservation.com/" },
        "2": { "textContent": "Rezervă. Perfect. Prin AI.", "className": "wp-block-site-title", "tagName": "P" },
        "3": { "textContent": "Primary Menu", "className": "primary-menu-container", "tagName": "NAV" },
        "4": { "textContent": "Rezervări turistice - Rezervări - Bull Shoals Lake, USA", "className": "wp-block-post-title", "tagName": "H1" }
    },
    "titledOnlineIDVersion": 0,
    "time": 1742619791045,
    "title": "Rezervări turistice - Rezervări - Bull Shoals Lake, USA 🚢✈️🗺️🚗🧳bilete avion, hotel, oferte, pachete.",
    "URL": "https://perfect-reservation.com/rezervari-bull-shoals-lake-usa/"
  },
  {
    "titledOnlineID": "guide_frontend_dev_blog",
    "HCA": "known",
    "headlines": {
        "0": { "textContent": "Modern Frontend Development", "className": "post-title", "tagName": "H1" },
        "1": { "textContent": "Introduction to React Hooks", "className": "post-subtitle", "tagName": "H2" },
        "2": { "textContent": "State management with Zustand", "className": "post-body-heading", "tagName": "H3" },
        "3": { "textContent": "Styling with Tailwind CSS", "className": "post-body-heading", "tagName": "H3" },
        "4": { "textContent": "Read more about our advanced patterns.", "className": "read-more", "tagName": "A", "href": "#" }
    },
    "titledOnlineIDVersion": 1,
    "time": 1742619899123,
    "title": "A Guide to Modern Frontend Development in 2024",
    "URL": "https://dev-blog.example.com/frontend-guide-2024"
  },
  {
    "titledOnlineID": "creativeai_future_of_art",
    "HCA": "unknown",
    "headlines": {
        "0": { "textContent": "AI in the Creative Sphere", "className": "article-main-title", "tagName": "H1" },
        "1": { "textContent": "Generative models like Imagen are changing the landscape for artists and designers.", "className": "article-intro", "tagName": "P" },
        "2": { "textContent": "Ethical Considerations", "className": "section-heading", "tagName": "H2" },
        "3": { "textContent": "The Future of Digital Art", "className": "section-heading", "tagName": "H2" },
        "4": { "textContent": "Contact us for collaborations.", "className": "contact-link", "tagName": "A", "href": "/contact" }
    },
    "titledOnlineIDVersion": 0,
    "time": 1742619999456,
    "title": "The Rise of AI in Creative Industries: A Deep Dive",
    "URL": "https://art-and-ai-magazine.com/rise-of-ai-creativity"
  }
];

const fetchMockResults = (query: string, collection: string | null): Promise<any[]> => {
  return new Promise((resolve) => {
    setTimeout(() => {
      if (!query && !collection) {
        return resolve([]);
      }

      const lowercasedQuery = query.toLowerCase();

      const filteredResults = MOCK_DATA.filter(item => {
        const titleMatch = item.title.toLowerCase().includes(lowercasedQuery);
        
        const headlineMatch = Object.values(item.headlines).some((headline: any) => 
            headline.textContent && headline.textContent.toLowerCase().includes(lowercasedQuery)
        );
        
        return titleMatch || headlineMatch;
      });

      resolve(filteredResults);
    }, 300);
  });
};

// === From: services/v1/searchApi.ts ===
const fetchApiResults = async (query: string, collection: string | null, config: SearchConfig): Promise<any[]> => {
  const { baseUrl, defaultXMediaWriteLocation, size } = config.api;

  const params = new URLSearchParams({
    from: '0',
    size: size.toString(),
  });

  if (query) {
    params.set('t', query);
  }

  if (collection) {
    params.set('h', collection);
  }

  let location;
  try {
    if (window.location.protocol === 'blob:' || !window.location.origin || window.location.origin === 'null') {
        location = defaultXMediaWriteLocation;
    } else {
        location = window.location.origin;
    }
  } catch (e) {
    location = defaultXMediaWriteLocation;
  }
  
  params.set('xmediawritelocation', location);
  
  const url = `${baseUrl}?${params.toString()}`;

  console.log(`Fetching from API: ${url}`);

  try {
    const response = await fetch(url, {
      method: 'GET',
      mode: 'cors',
    });

    if (!response.ok) {
      throw new Error(`API request failed with status ${response.status}: ${response.statusText}`);
    }

    const data = await response.json();
    
    if (Array.isArray(data)) {
        return data;
    } else {
        console.warn("API response was not an array:", data);
        return [];
    }
  } catch (error: unknown) {
    const origin = window.location.origin;
    const errorMessage = `Failed to fetch from search API. This is often a CORS issue. Your server needs to allow requests from this origin: ${origin}`;
    
    console.error(errorMessage);
    console.error("Full error:", error);

    if (error instanceof Error && error.message.includes('API request failed')) {
        throw error;
    }
    
    throw new Error(errorMessage);
  }
};


// === From: hooks/useSearchEngine.ts ===
const useSearchEngine = (config: SearchConfig | null) => {
  const [query, setQuery] = useState<string>('');
  const [collection, setCollection] = useState<string | null>(null);
  const [results, setResults] = useState<any[] | null>(null);
  const [isLoading, setIsLoading] = useState<boolean>(false);
  const [error, setError] = useState<string | null>(null);
  const [hasSearched, setHasSearched] = useState(false);

  const debounceTimeoutRef = useRef<number | null>(null);
  const searchRequestRef = useRef(0);

  const runSearch = useCallback(async (searchQuery: string, searchCollection: string | null, searchId: number) => {
    if (!config) return;
    if (!searchQuery.trim() && !searchCollection) {
        setResults(null);
        setIsLoading(false);
        return;
    };

    if (!hasSearched) setHasSearched(true);
    setIsLoading(true);
    setError(null);
    setResults([]);

    setQuery(searchQuery); 
    setCollection(searchCollection);

    const params = new URLSearchParams();
    if (searchQuery) params.set('t', searchQuery);
    if (searchCollection) params.set('h', searchCollection);
    
    if (window.location.protocol !== 'blob:') {
        const newUrl = params.toString() ? `${window.location.pathname}?${params.toString()}` : window.location.pathname;
        window.history.pushState({}, '', newUrl);
    }

    try {
        if (config.searchMode === 'mock') {
            const fetchedResults = await fetchMockResults(searchQuery, searchCollection);
            if (searchId === searchRequestRef.current) setResults(fetchedResults);
        } else if (config.searchMode === 'api') {
            const fetchedResults = await fetchApiResults(searchQuery, searchCollection, config);
            if (searchId === searchRequestRef.current) setResults(fetchedResults);
        }
    } catch (err: unknown) {
      if (searchId === searchRequestRef.current) {
         if (err instanceof Error) {
           setError(`An error occurred: ${err.message}`);
         } else {
           setError('An unknown error occurred.');
         }
      }
    } finally {
      if (searchId === searchRequestRef.current) {
        setIsLoading(false);
      }
    }
  }, [config, hasSearched]);

  useEffect(() => {
    if (!config) return;

    const params = new URLSearchParams(window.location.search);
    const queryFromUrl = params.get('t');
    const collectionFromUrl = params.get('h');

    if (queryFromUrl !== null || collectionFromUrl !== null) {
      const initialQuery = queryFromUrl || '';
      searchRequestRef.current += 1;
      runSearch(initialQuery, collectionFromUrl, searchRequestRef.current);
    }
  }, [config, runSearch]);

  useEffect(() => {
    if (!config) return;
    
    const params = new URLSearchParams(window.location.search);
    const initialQuery = params.get('t') || '';
    const initialCollection = params.get('h');

    if (query === initialQuery && collection === initialCollection && !hasSearched) {
      return;
    }

    if (debounceTimeoutRef.current) {
      clearTimeout(debounceTimeoutRef.current);
    }

    debounceTimeoutRef.current = window.setTimeout(() => {
      if (query.trim() || collection) {
        searchRequestRef.current += 1;
        runSearch(query, collection, searchRequestRef.current);
      } else {
        setResults(null);
        setHasSearched(false);
        if (window.location.protocol !== 'blob:') {
          window.history.pushState({}, '', window.location.pathname);
        }
      }
    }, config.searchDebounceMilliseconds);

    return () => {
      if (debounceTimeoutRef.current) {
        clearTimeout(debounceTimeoutRef.current);
      }
    };
  }, [query, collection, config, runSearch, hasSearched]);

  return {
    query,
    setQuery,
    collection,
    results,
    isLoading,
    error,
    hasSearched
  };
};

// --- App specific getAppOrigin ---
// This function is used by Auth0 components to determine the correct callback URL.
const getAppOrigin = (): string => {
  const safeOrigin = 'http://127.0.0.1:5500';
  try {
    if (window.location.protocol === 'blob:' || !window.location.origin || window.location.origin === 'null') {
      return safeOrigin;
    }
    return window.location.origin;
  } catch (e) {
    return safeOrigin;
  }
};


// === From: components/AuthNav.tsx ===
const AuthNav: React.FC = () => {
  const {
    isLoading,
    isAuthenticated,
    loginWithRedirect,
    logout,
  } = useAuth0();

  if (isLoading) {
    return <div className="h-8 w-24 bg-gray-200 animate-pulse"></div>;
  }

  const loginButtonClasses = "px-4 py-2 text-sm font-medium text-white bg-blue-600 hover:bg-blue-700 focus:outline-none focus:ring-2 focus:ring-offset-2 focus:ring-blue-500";
  const secondaryButtonClasses = "text-sm font-medium text-slate-600 hover:text-blue-600 hover:underline";
  
  return (
    <div className="flex items-center gap-4">
      {!isAuthenticated ? (
        <>
          <button
            onClick={() => loginWithRedirect({
              authorizationParams: {
                screen_hint: 'signup',
              },
            })}
            className={secondaryButtonClasses}
          >
            Sign Up
          </button>
          <button
            onClick={() => loginWithRedirect()}
            className={loginButtonClasses}
          >
            Log In
          </button>
        </>
      ) : (
        <>
          <a href="/account" className={`${secondaryButtonClasses} flex items-center gap-2`}>
            <UserIcon />
            My Account
          </a>
          <button
            onClick={() => logout({ logoutParams: { returnTo: getAppOrigin() } })}
            className={secondaryButtonClasses}
          >
            Log Out
          </button>
        </>
      )}
    </div>
  );
};

// === From: components/Header.tsx ===
const Header: React.FC = () => {
  return (
    <header className="w-full border-b border-gray-200">
      <div className="w-full max-w-4xl mx-auto flex items-center justify-between p-4">
        <div className="flex items-center gap-6">
          <a href="/" className="flex items-center gap-3">
              <HeartIcon />
          </a>
          <nav>
              <a href="/about" className="text-slate-600 hover:text-blue-600 hover:underline">About</a>
          </nav>
        </div>
        <AuthNav />
      </div>
    </header>
  );
};


// === From: pages/AboutPage.tsx ===
const AboutPage: React.FC = () => {
  return (
    <main className="w-full max-w-4xl mx-auto p-4 sm:p-6 lg:p-8">
      <div className="prose prose-slate max-w-none">
        <h1>About This Application</h1>
        <p>
          This is an advanced content generation and search assistant, built with modern web technologies. 
          It leverages the power of AI to provide insightful and relevant information.
        </p>
        <p>
          Our mission is to create a seamless and intelligent experience for discovering and interacting with content.
        </p>
      </div>
    </main>
  );
};

// === From: pages/AccountPage.tsx ===
const AccountPage: React.FC = () => {
  const { user } = useAuth0();

  if (!user) {
    return null; 
  }

  return (
    <main className="w-full max-w-4xl mx-auto p-4 sm:p-6 lg:p-8">
      <div className="flex flex-col sm:flex-row items-center sm:items-start gap-6">
        <img
          src={user.picture}
          alt={user.name}
          className="w-24 h-24 rounded-full border-4 border-slate-200"
        />
        <div className="text-center sm:text-left">
          <h1 className="text-3xl font-bold text-slate-900">{user.name}</h1>
          <p className="text-lg text-slate-600 mt-1">{user.email}</p>
        </div>
      </div>
      <div className="mt-8 pt-6 border-t border-slate-200">
        <h2 className="text-xl font-semibold text-slate-800">Profile Details</h2>
        <div className="mt-4 bg-slate-50 p-4 overflow-x-auto">
          <pre className="text-sm">
            <code>{JSON.stringify(user, null, 2)}</code>
          </pre>
        </div>
      </div>
    </main>
  );
};

const AuthenticatedAccountPage = withAuthenticationRequired(AccountPage, {
  onRedirecting: () => (
    <div className="min-h-screen flex items-center justify-center">
        <p className="text-slate-500 text-lg">Redirecting to login...</p>
    </div>
  ),
});

// === From: components/SearchInput.tsx ===
interface SearchInputProps {
  query: string;
  setQuery: (query: string) => void;
  isLoading: boolean;
  config: SearchConfig['searchInput'];
}

const SearchInput: React.FC<SearchInputProps> = ({ query, setQuery, isLoading, config }) => {
  return (
    <div className="w-full relative">
       <div className="absolute inset-y-0 left-0 pl-3 flex items-center pointer-events-none">
          <SearchIcon />
      </div>
      <input
        type="text"
        value={query}
        onChange={(e) => setQuery(e.target.value)}
        placeholder={config.placeholder}
        className="w-full h-full pl-10 pr-4 py-3 bg-white border border-gray-400 text-slate-900 placeholder-slate-500 focus:outline-none focus:ring-1 focus:ring-blue-500 text-base"
        aria-label="Search input"
      />
    </div>
  );
};
const MemoizedSearchInput = React.memo(SearchInput);


// === From: components/HeadlinesDisplay.tsx ===
interface HeadlinesDisplayProps {
  headlines: any;
  config: SearchConfig;
  searchMode: 'mock' | 'api';
}

const HeadlinesDisplay: React.FC<HeadlinesDisplayProps> = ({ headlines, config, searchMode }) => {
  const [isExpanded, setIsExpanded] = useState(false);
  
  const isApiOrMockMode = searchMode === 'api' || searchMode === 'mock';
  const renderConfig = config.api.resultsConfigs;
  const mockConfig = config.headlines;

  const getDetailsArray = (hl: any): any[] => {
    if (!hl) return [];
    if (typeof hl === 'object' && !Array.isArray(hl)) {
        if (hl.details && Array.isArray(hl.details)) {
            return hl.details;
        }
        return Object.values(hl);
    }
    return [];
  };

  const details = getDetailsArray(headlines);
  const title = isApiOrMockMode ? null : mockConfig.title;

  const filteredDetails = isApiOrMockMode
    ? details.filter((detail: any) => 
        detail && typeof detail === 'object' && !renderConfig.headlinesFilterForScreen.some(filterText => 
          Object.values(detail).some(val => typeof val === 'string' && val.includes(filterText))
        )
      )
    : details;
  
  const limit = isApiOrMockMode ? renderConfig.headingsLastLimit : 3;
  const hasMore = filteredDetails.length > limit;
  const displayedDetails = isExpanded ? filteredDetails : filteredDetails.slice(0, limit);

  const renderApiHeadline = (detail: any, index: number) => {
    const propertiesToShow = Object.entries(detail)
        .filter(([key]) => renderConfig.headlinesShow.includes(key));
    if (propertiesToShow.length === 0) return null;

    return (
        <li key={index} className="pl-4 border-l-2 border-gray-200">
             <div className="flex flex-col space-y-1">
                {propertiesToShow.map(([key, value]) => (
                     <div key={key} className="flex">
                        <strong className="font-medium text-gray-500 mr-2 w-28 shrink-0 capitalize">{key.replace(/([A-Z])/g, ' $1').trim()}:</strong>
                        <span className="break-words">{String(value)}</span>
                     </div>
                ))}
            </div>
        </li>
    );
  };
    
  const renderMockHeadline = (detail: any, index: number) => (
      <li key={index} className="list-disc ml-5">
        <span className="break-words">{detail.text}</span>
      </li>
  );

  return (
    <div>
      {title && <h3 className="text-md font-semibold text-slate-700 mb-2">{title}</h3>}
      <ul className="space-y-4 text-sm text-gray-700">
        {displayedDetails.map(isApiOrMockMode ? renderApiHeadline : renderMockHeadline)}
      </ul>
      {hasMore && !isExpanded && (
        <button
          onClick={() => setIsExpanded(true)}
          className="text-blue-600 hover:underline text-sm mt-4"
        >
          {mockConfig.expandText.replace('{count}', filteredDetails.length.toString())}
        </button>
      )}
    </div>
  );
};

// === From: components/ResultItem.tsx ===
interface ResultItemProps {
  item: any;
  config: SearchConfig;
  searchMode: 'mock' | 'api';
}

const ResultItem: React.FC<ResultItemProps> = ({ item, config, searchMode }) => {
  const isApiOrMockMode = searchMode === 'api' || searchMode === 'mock';
  const renderConfig = config.api.resultsConfigs;
  const title = item.title || item.Title || "Untitled";

  const renderApiDetails = () => {
    return (
      <div className="text-sm text-gray-600 space-y-1 mt-2">
        {Object.entries(item).map(([key, value]) => {
          if (renderConfig.titledOnlineIDDoNotShow.includes(key) || typeof value === 'object' && value !== null) {
            return null;
          }
          return (
            <div key={key}>
              <span className="font-semibold capitalize">{key}:</span>
              <span className="ml-2">{String(value)}</span>
            </div>
          );
        })}
      </div>
    );
  };

  return (
    <article className="py-8">
      <header>
        <h2 className="text-xl font-semibold text-blue-700 mb-1">{title}</h2>
        {isApiOrMockMode ? renderApiDetails() : (
          <div className="text-sm text-gray-500 space-x-2">
              <span>Source: {item.provider}</span>
              {item.updatedAt && <><span>|</span><span>Update (from site): {item.updatedAt}</span></>}
          </div>
        )}
      </header>
      {(item.headlines || item.Headlines) && (
        <div className="mt-4">
          <HeadlinesDisplay 
            headlines={item.headlines || item.Headlines} 
            config={config}
            searchMode={searchMode}
            />
        </div>
      )}
    </article>
  );
};
const MemoizedResultItem = React.memo(ResultItem);

// === From: components/ResultDisplay.tsx ===
interface ResultDisplayProps {
  results: any[] | null;
  isLoading: boolean;
  error: string | null;
  config: SearchConfig;
  searchMode: 'mock' | 'api';
}

const SkeletonLoader: React.FC = () => {
    const commonSkeleton = (
        <div className="space-y-4">
            <div className="h-5 bg-slate-200 w-3/4"></div>
            <div className="h-4 bg-slate-200 w-1/2"></div>
            <div className="space-y-2 pt-2">
                <div className="h-4 bg-slate-200 w-full"></div>
                <div className="h-4 bg-slate-200 w-5/6"></div>
                <div className="h-4 bg-slate-200 w-full"></div>
            </div>
        </div>
    );

    return (
        <div className="space-y-8 animate-pulse pt-6">
            {[...Array(2)].map((_, i) => <div key={i}>{commonSkeleton}</div>)}
        </div>
    );
};


const ResultDisplay: React.FC<ResultDisplayProps> = ({ results, isLoading, error, config, searchMode }) => {
  if (error) {
    return (
      <div className="bg-red-50 border border-red-200 text-red-700 px-4 py-3 mt-4" role="alert">
        <strong className="font-bold">Error: </strong>
        <span className="block sm:inline">{error}</span>
      </div>
    );
  }

  const noResults = !results || results.length === 0;

  if (isLoading && noResults) {
    return <SkeletonLoader />;
  }

  if (!isLoading && noResults) {
    return (
      <div className="text-center py-16">
        <p className="text-slate-500 text-base">{config.results.noResultsText}</p>
      </div>
    );
  }

  return (
    <div className="divide-y divide-slate-200">
      {results?.map((item, index) => (
        <MemoizedResultItem 
            key={`${item.time || item.URL}-${index}`} 
            item={item} 
            config={config}
            searchMode={searchMode} 
        />
      ))}
    </div>
  );
};
const MemoizedResultDisplay = React.memo(ResultDisplay);


// === From: components/SearchContext.tsx ===
interface SearchContextProps {
  query: string;
  collection: string | null;
  resultCount: number;
  isLoading: boolean;
  config: SearchConfig['searchContext'];
  searchMode: 'mock' | 'api';
}

const SearchContext: React.FC<SearchContextProps> = ({ query, collection, resultCount, isLoading, config, searchMode }) => {
  const hasQuery = query && query.trim() !== '';

  if (!hasQuery && !collection) {
    return null;
  }

  return (
    <div className="py-4 my-4 border-t border-b border-gray-200">
      <div className="space-y-1 text-base">
        {hasQuery && (
          <p className="text-gray-500">
            {config.queryLabel.replace('{query}', `"${query}"`)}
          </p>
        )}
        {collection && (
          <p className="text-gray-500">
            {config.collectionLabel}{' '}
            <a href={`/?h=${collection}&t=${query}`} className="text-blue-600 hover:underline font-semibold">#{collection}</a>
          </p>
        )}
        {!isLoading && (
          <p className="text-gray-500 pt-2">{config.resultsLabel.replace('{count}', resultCount.toString())}</p>
        )}
      </div>
    </div>
  );
};


// === From: App.tsx ===
const SearchPage: React.FC<{config: SearchConfig}> = ({ config }) => {
  const {
    query,
    setQuery,
    collection,
    results,
    isLoading,
    error,
    hasSearched
  } = useSearchEngine(config);

  const showHeaderTitle = !hasSearched && !collection;

  return (
    <main className="w-full max-w-4xl mx-auto flex flex-col flex-grow p-4 sm:p-6 lg:p-8">
      {showHeaderTitle && (
          <div className="text-center my-16">
            <h1 className="text-4xl sm:text-5xl font-bold text-slate-900 mb-4">
              {config.appTitle}
            </h1>
          </div>
      )}
      
      <div 
        className={
          !showHeaderTitle
          ? 'sticky top-4 z-10 bg-white/80 backdrop-blur-sm -mx-4 px-4 pt-2 pb-4 mb-4'
          : ''
        }
      >
        <MemoizedSearchInput
          query={query}
          setQuery={setQuery}
          isLoading={isLoading}
          config={config.searchInput}
        />
      </div>
      
      {(hasSearched || collection) && (
        <div className="flex-grow">
          <SearchContext
            query={query}
            collection={collection}
            resultCount={results?.length ?? 0}
            isLoading={isLoading}
            config={config.searchContext}
            searchMode={config.searchMode}
          />
          <MemoizedResultDisplay
            results={results}
            isLoading={isLoading}
            error={error}
            config={config}
            searchMode={config.searchMode}
          />
        </div>
      )}
    </main>
  );
};

const App: React.FC = () => {
  const [config, setConfig] = useState<SearchConfig | null>(null);
  const [configError, setConfigError] = useState<string | null>(null);
  
  useEffect(() => {
    const loadConfig = async () => {
      try {
        const response = await fetch('./config/search.json');
        if (!response.ok) {
          throw new Error(`HTTP error! status: ${response.status}`);
        }
        const configData: SearchConfig = await response.json();
        setConfig(configData);
      } catch (e) {
        console.error("Failed to load app configuration:", e);
        setConfigError("Could not load application configuration. Please try again later.");
      }
    };
    loadConfig();
  }, []);
  
  const renderPage = () => {
    const path = window.location.pathname;

    if (!config) {
      return (
        <div className="min-h-screen bg-white flex items-center justify-center">
          <div className="text-center">
             {configError ? (
               <div className="bg-red-50 border border-red-200 text-red-700 px-4 py-3" role="alert">
                 <strong className="font-bold">Error: </strong>
                 <span className="block sm:inline">{configError}</span>
               </div>
             ) : (
              <p className="text-slate-500 text-lg">Loading...</p>
             )}
          </div>
        </div>
      );
    }

    switch (path) {
      case '/account':
        return <AuthenticatedAccountPage />;
      case '/about':
        return <AboutPage />;
      default:
        return <SearchPage config={config} />;
    }
  };

  return (
    <div className="min-h-screen bg-white text-slate-800 font-serif flex flex-col">
      <Header />
      {renderPage()}
    </div>
  );
};


//======================================================================
// APPLICATION ENTRY POINT
//======================================================================

const AuthProviderWrapper: React.FC = () => {
  const [authConfig, setAuthConfig] = useState<AuthConfig | null>(null);
  const [error, setError] = useState<string | null>(null);

  useEffect(() => {
    const fetchAuthConfig = async () => {
      try {
        const response = await fetch('./config/auth.json');
        if (!response.ok) {
          throw new Error('Could not fetch auth configuration.');
        }
        const config: AuthConfig = await response.json();

        if (!config.domain || config.domain === 'YOUR_AUTH0_DOMAIN' || !config.clientId || config.clientId === 'YOUR_AUTH0_CLIENT_ID') {
          throw new Error('Auth0 is not configured. Please add your domain and client ID to public/config/auth.json.');
        }
        setAuthConfig(config);
      } catch (e: any) {
        console.error("Auth config error:", e);
        setError(e.message);
      }
    };
    fetchAuthConfig();
  }, []);

  if (error) {
    return (
      <div className="min-h-screen bg-white flex items-center justify-center p-4">
        <div className="bg-red-50 border border-red-200 text-red-700 px-4 py-3 text-center" role="alert">
           <strong className="font-bold">Configuration Error:</strong>
           <p>{error}</p>
           <p className="mt-2 text-sm">Please follow the setup instructions in the README to configure authentication.</p>
        </div>
      </div>
    );
  }

  if (!authConfig) {
    return (
        <div className="min-h-screen bg-white flex items-center justify-center">
            <p className="text-slate-500 text-lg">Loading Authentication...</p>
        </div>
    );
  }

  return (
    <Auth0Provider
      domain={authConfig.domain}
      clientId={authConfig.clientId}
      authorizationParams={{
        redirect_uri: getAppOrigin()
      }}
    >
      <App />
    </Auth0Provider>
  );
};

const rootElement = document.getElementById('root');
if (!rootElement) {
  throw new Error("Could not find root element to mount to");
}

const root = ReactDOM.createRoot(rootElement);
root.render(
  <React.StrictMode>
    <AuthProviderWrapper />
  </React.StrictMode>
);